from rdflib import Graph
from SPARQLWrapper import SPARQLWrapper, XML, JSON
import requests
import json


def query_with_in_scheme(concept_scheme):
    if concept_scheme != None:
        in_scheme_triple = "?concept skos:inScheme <%(concept_scheme)s>"%dict(concept_scheme=concept_scheme)
    else: in_scheme_triple = ''
    return """
            PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
            select ?concept ?prefLabel (GROUP_CONCAT(?broaderPrefLabel;SEPARATOR="|") as ?path) (count(?broader) as ?counter) 
            where {
                ?concept a skos:Concept ;
                         skos:prefLabel ?prefLabel .
                %(in_scheme_triple)s         
                OPTIONAL { ?concept skos:broader+ ?broader .
                           ?broader skos:prefLabel ?broaderPrefLabel .
                        }
                }
                group by ?concept ?prefLabel
            """%dict(in_scheme_triple=in_scheme_triple)

def skos_choices_sparql_helper(field):
    '''Return a list of the concepts of a concept scheme served from a SPARQL endpoint'''

    sparql_endpoint = field['skos_choices_sparql_endpoint']
    is_poolparty = field.get('skos_choices_is_poolparty', False)
    concept_scheme = field.get('skos_choices_concept_scheme', None)

    query = query_with_in_scheme(concept_scheme)
    if is_poolparty:
        response = requests.post(sparql_endpoint, data={"query": query, "format": "application/json"}, headers={ "Content-Type": "application/x-www-form-urlencoded"}).content.decode("utf-8")
        results = json.loads(response)
    else:
        sparql = SPARQLWrapper(sparql_endpoint)
        sparql.setQuery(query)
        sparql.setReturnFormat(JSON)
        try:
            results = sparql.queryAndConvert()
        except Exception as e:
            print(e)
    if results['results']['bindings']:
        bindings = results['results']['bindings']
        skos_choices = []
        for binding in bindings:
            path = (binding['path']['value'].split("|"))
            prefLabel = binding['prefLabel']['value']
            if path[0] != '':
                path.reverse()
                path.append(prefLabel)
                label = ' -> '.join(path)
            else:
                label = prefLabel
            choice = {
                'value': binding['concept']['value'],
                'label': label
            }
            skos_choices.append(choice)
    ordered_choices = sorted(skos_choices, key=lambda d: d['label'])
    return ordered_choices



import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit
from ckanext.vocabularies.helpers import skos_choices_sparql_helper




class VocabulariesPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IConfigurer)

    # Declare that this plugin will implement ITemplateHelpers.
    plugins.implements(plugins.ITemplateHelpers)

    # IConfigurer

    def update_config(self, config_):
        toolkit.add_template_directory(config_, 'templates')
        toolkit.add_public_directory(config_, 'public')
        toolkit.add_resource('assets',
            'vocabularies')


    def get_helpers(self):
        '''Register the most_popular_groups() function above as a template
        helper function.

        '''
        # Template helper function names should begin with the name of the
        # extension they belong to, to avoid clashing with functions from
        # other extensions.
        return {'skos_vocabulary_helper': skos_choices_sparql_helper}
